package jp.co.sds_corp.util.charset;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.nio.charset.Charset;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

public class CharsetChecker {
	static final CmnLogger logger = CmnLogFactory.getLogger();

	public void check(String filename, String charsetNameIn, String charsetNameOut) throws IOException {
		Charset charsetIn = Charset.forName(charsetNameIn);
		Charset charsetOut = Charset.forName(charsetNameOut);
		check(filename, charsetIn, charsetOut);
	}

	public void check(String filename, Charset charsetIn, Charset charsetOut) throws IOException {
		FileInputStream is = null;
		try {
			is = new FileInputStream(filename);
			check(is, charsetIn, charsetOut);

		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (Exception ex) {
					logger.error(this, "close error.", ex);
				}
			}
		}
	}

	public void check(InputStream in, Charset charsetIn, Charset charsetOut) throws IOException {
		Charset wrapCharsetIn = new CharsetWrapper(charsetIn);
		Charset wrapCharsetOut = new CharsetWrapper(charsetOut);

		InputStreamReader isr = new InputStreamReader(in, wrapCharsetIn);
		BufferedReader br = new BufferedReader(isr);
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		OutputStreamWriter osr = new OutputStreamWriter(bos, wrapCharsetOut);
		PrintWriter out = new PrintWriter(osr);
		int lineNo = 0;
		try {
			for ( ;; ) {
				lineNo++;
				String line = br.readLine();
				if (line == null) {
					break;
				}
				out.println(line);
				bos.reset();
			}
			out.flush();
			out.close();
		} catch (CharsetConvertException ex) {
			throw(new CharsetConvertException("convert error. line=" + lineNo, ex));
		} finally {
			try {
				br.close();
			} catch (Exception ex) {
				logger.error(this, "close error.", ex);
			}
			try {
				out.close();
			} catch (Exception ex) {
				logger.error(this, "close error.", ex);
			}
		}
	}
}
