package jp.co.sds_corp.util.charset;

import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CoderResult;

public class CharsetEncoderWrapper extends CharsetEncoder {
	CharsetEncoder encoder;
	Method encodeLoop;

	public CharsetEncoderWrapper(CharsetEncoder org) {
		super(org.charset(), org.averageBytesPerChar(), org.maxBytesPerChar());
		this.encoder = org;
		try {
			Class<?> targetClass = encoder.getClass();
			for ( ;; ) {
                try {
					encodeLoop = targetClass.getDeclaredMethod("encodeLoop", CharBuffer.class, ByteBuffer.class);
                    break;
                } catch (NoSuchMethodException ex) {
                    targetClass = targetClass.getSuperclass();
                    if (targetClass == null) {
                        throw(ex);
                    }
                }
			}
			encodeLoop.setAccessible(true);
		} catch (Exception ex) {
			throw(new RuntimeException("get method error.", ex));
		}
	}

	protected CoderResult encodeLoop(CharsetEncoder encoder, CharBuffer in, ByteBuffer out) {
		CoderResult result;
		try {
			result = (CoderResult )encodeLoop.invoke(encoder, in, out);
		} catch (Exception ex) {
			throw(new RuntimeException("invoke error.", ex));
		}
		return result;
	}

	@Override
	protected CoderResult encodeLoop(CharBuffer in, ByteBuffer out) {
		CoderResult result = encodeLoop(encoder, in, out);
		if (result.isMalformed() || result.isUnmappable()) {
			int postLen = 10;
			if (in.length() < postLen) {
				postLen = in.length();
			}
			int pos = in.position();
			int preLen = 10;
			if ((pos - preLen) < 0) {
				preLen = pos;
			}
			in.position(pos - preLen);
			CharBuffer pre = in.subSequence(0, preLen);
			CharBuffer target = in.subSequence(preLen, preLen + result.length());
			CharBuffer post = in.subSequence(preLen + result.length(), preLen + postLen);
			throw(new CharsetConvertException("encode error. " + result
					+ " (" + pos + ":" + result.length() + ")"
					+ " \"" + pre + "[" + target + "]" + post + "\""));
		}
		return result;
	}

}
