package	jp.co.sds_corp.util.common;

import java.util.Arrays;

/**
* 文字列バッファ操作を補助します。
* @author	小見山　聡
*/
public class CmnStringBuffer
{
	private	StringBuffer	buf = null;

	/**
	* 文字列バッファを初期化します。
	*/
	public CmnStringBuffer() {
		this( 16 );
	}

	/**
	* 文字列バッファを指定された初期容量で初期化します。
	* @param	nLen		初期容量
	*/
	public CmnStringBuffer( int nLen ) {
		buf = new StringBuffer( nLen );
	}

	/**
	* 文字列バッファを指定された文字列で初期化します。
	* @param	str			文字列
	*/
	public CmnStringBuffer( String str ) {
		buf = new StringBuffer( str );
	}

	/**
	* 文字列バッファに文字列を追加します。
	* @param	str			追加する文字列
	* @return	追加後の文字列バッファ
	*/
	public CmnStringBuffer append( String str ) {
		buf.append( str );
		return this;
	}

	/**
	* 文字列バッファに文字列を追加します。
	* この時、改行コードも追加します。
	* @param	str			追加する文字列
	* @return	追加後の文字列バッファ
	*/
	public CmnStringBuffer appendln( String str ) {
		buf.append( str );
		buf.append( "\n" );
		return this;
	}

	/**
	* 文字列バッファの長さを指定します。
	* @param	nLen		バッファの長さ
	*/
	public void setLength( int nLen ) {
		setLength( nLen, true );
	}

	/**
	* 文字列バッファの長さを指定します。
	* Append フラグが指定されている場合、スペースを後ろに追加します。
	* @param	nLen		バッファの長さ
	* @param	bAppend		Appendフラグ
	*/
	public void setLength( int nLen, boolean bAppend ) {
		if ( nLen > buf.length()) {
			char[] cSp = new char[nLen];
			Arrays.fill( cSp, ' ' );
			if ( bAppend ) {
				buf.append( cSp, 0, nLen - buf.length() );
			} else {
				buf.insert( 0, cSp, 0, nLen - buf.length() );
			}
		} else {
			buf.setLength( nLen );
		}
	}

	/**
	* 文字列バッファから文字を削除します。
	* @param	start		開始インデックス
	* @param	end			終了インデックス
	* @return	削除後の文字列バッファ
	*/
	public CmnStringBuffer delete( int start, int end ) {
		buf.delete( start, end );
		return this;
	}

	/**
	* 文字列バッファの長さを返します。
	* @return	文字列バッファの長さ
	*/
	public int length() {
		return buf.length();
	}

	/**
	* 文字列バッファの内容を文字列として返します。
	* @return	文字列バッファの内容
	*/
	public String toString() {
		return buf.toString();
	}
}
