package jp.co.sds_corp.util.common;

import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;
import java.util.Vector;

/**
* Servlet経由でWebサーバにファイルをアップロードします。
* @author	杉田　健児
*/
public class ServletUploader
{
	private String	servletName;

	/**
	* 指定されたサーブレット名で初期化します。
	* @param	servletName	サーブレット名
	*/
	public ServletUploader( String servletName ) {
		this.servletName = servletName;
	}

	/**
	* サーブレットにアップロード要求を送信します。
	* @param	type		ファイル種別(txt,property)
	* @param	fileName	ファイル名
	* @param	data		アップロードする内容
	*/
	public URLConnection uploadServletFile( String type, String fileName,
											Object data )
		throws Exception
	{
		URL url = new URL( servletName );
		URLConnection ucon = url.openConnection();
		ucon.setDoOutput( true );
		ucon.setUseCaches( false );
		ucon.setDoInput( true );

		Vector<Object> vec = new Vector<Object>();
		vec.addElement( type );
		vec.addElement( fileName );
		vec.addElement( data );

		ObjectOutputStream oos =
					new ObjectOutputStream( ucon.getOutputStream() );
		oos.writeObject( vec );
		oos.flush();
		oos.close();

		return ucon;
	}


	/**
	* サーブレットにテキストファイルをアップロードします。
	* @param	fileName	ファイル名
	* @param	data		アップロードする内容
	*/
	public boolean uploadTextFile( String fileName, String data )
		throws Exception
	{
		URLConnection ucon = uploadServletFile( "txt", fileName, data );
		InputStream instream = ucon.getInputStream();
		if ( !checkStatus( instream ) ) {
			instream.close();
			return false;
		}
		instream.close();
		return true;
	}

	/**
	* サーブレットにPropertyファイルをアップロードします。
	* @param	fileName	ファイル名
	* @param	prop		アップロードする内容
	*/
	public boolean uploadPropertyFile( String fileName, Properties prop )
		throws Exception
	{
		URLConnection ucon = uploadServletFile( "property", fileName, prop );
		InputStream instream = ucon.getInputStream();
		if ( !checkStatus( instream ) ) {
			instream.close();
			return false;
		}
		instream.close();
		return true;
	}

	/**
	* サーブレットからの応答をチェックします
	* @param	instream	サーブレットからの応答Stream
	* @return	true:正常 false:エラー
	*/
	public boolean checkStatus( InputStream instream )
		throws Exception
	{
		byte sts[] = new byte[16];
		int sts_len = 3;
		String sts_str = "";
		int len;
		int total = 0;
		for( ; total < sts_len; ) {
			len = instream.read( sts, 0, (sts_len - total) );
			total += len;
			if( len == -1 ) {
				break;
			}
			sts_str = sts_str.concat( new String( sts, 0, len ) );
		}
		if( ( new String( sts, 0, 3 )).equals( "000" ) ) {
			return true;
		}
		return false;
	}
}
