package	jp.co.sds_corp.util.config;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
* プロパティファイルから設定情報を読み出します。
*/
public class PropertiesConfiguration extends AbstractConfiguration
{
	private static final long serialVersionUID = 1L;
	protected File              file;
    protected Properties        properties = new Properties();
    protected String            current = "";

    public PropertiesConfiguration( String path ) throws IOException {
        this( new File( path ));
    }
    public PropertiesConfiguration( File file ) throws IOException {
        this.file = file;
        properties.load( new BufferedInputStream( new FileInputStream( file )));
    }

    public IConfiguration getParent() {
        if ( current.length() == 0 ) {
            throw new RuntimeException( "current is toplevel." );
        }
        int index = current.lastIndexOf( '.', current.length() - 1 );
        if ( index == -1 ) {
            index = 0;
        }
        try {
            PropertiesConfiguration clone = (PropertiesConfiguration )clone();
            clone.current = clone.current.substring( 0, index );
            return clone;
        } catch ( CloneNotSupportedException ex ) {
            throw new InternalError();
        }
    }

    public IConfiguration getChild( String name ) {
        try {
            PropertiesConfiguration clone = (PropertiesConfiguration )clone();
            clone.current += name + '.';
            return clone;
        } catch ( CloneNotSupportedException ex ) {
            throw new InternalError();
        }
    }

    public IConfiguration getAttributes() {
        return getChild( "attribute" );
    }

    public String getName() {
        if ( current.length() == 0 ) {
            return file.getName();
        }
        return current.substring( 0, current.length() - 1 );
    }

    /** */
    public String get() throws UnspecifiedPropertyException {
        String value = (String )properties.get( current );
        if ( value == null ) {
            throw new UnspecifiedPropertyException( current );
        }
        return value;
    }
    public String get( String key )
    	throws UnspecifiedPropertyException
	{
        String value = (String )properties.get( current + key );
        if ( value == null ) {
            throw new UnspecifiedPropertyException( current + key );
        }
        return value;
    }

}
