package	jp.co.sds_corp.util.db;

import java.io.Serializable;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
* カラム情報保持クラス
* @author 小見山　聡
*/
public class DbColumn implements Serializable, Cloneable
{
	private static final long serialVersionUID = 1L;

	// フィールド ---------------------------------------------------------
	/** カラム名 */
	private String szName = null;
	/** 表示名 */
	private String szLabel = null;
	/** Javaクラス */
	private String szClassName = null;
	/** 型名 */
	private String szTypeName = null;
	/** 型 */
	private int nType = 0;

	/** 列を表示するのに必要な最大文字数 */
	private int nSize = 0;
	/** 精度 */
	private int nPrec = 0;
	/** 小数点以下桁数 */
	private int nScale = 0;
	/** NULL値を許すか否か（0:NULL値を許さない, 1:NULL値を許す, 2:不明） */
	private int nNull;
	/** テーブル名 */
	private String szTbl = null;

	/** データオブジェクト */
	private Object	obj = null;


// フィールド取得メソッド ----------------------------------------------

	/** カラム名取得 */
	public String getName() {
		return szName;
	}

	/** 表示名取得 */
	public String getLabel() {
		return szLabel;
	}

	/** Javaクラス名取得 */
	public String getClassName() {
		return szClassName;
	}

	/** カタログ名取得 */
	public String getTypeName() {
		return szTypeName;
	}

	/** 型取得 */
	public int getType() {
		return nType;
	}

	/** 列を表示するのに必要な最大文字数を取得する */
	public int getSize() {
		return nSize;
	}

	/** 精度取得 */
	public int getPrecision() {
		return nPrec;
	}

	/** 小数点以下桁数取得 */
	public int getScale() {
		return nScale;
	}

	/** NULL値を許すか否か */
	public int isNullable() {
		return nNull;
	}

	/** カラムのオブジェクトの設定 */
	public void setObject( Object obj ) {
		this.obj = obj;
	}

	/** カラムのオブジェクトの取得 */
	public Object getObject() {
		return obj;
	}



// コンストラクタ ---------------------------------------------------

	/** コンストラクタ
	* @param	columnName	カラム名
	* @param	columnLabel	表示名
	* @param	columnType	型
	* @param	columnDisplaySize	列を表示するのに必要な最大文字数
	* @param	precision	精度
	* @param	scale		小数点以下桁数
	* @param	nullable	NULL値を許すか否か（0:NULL値を許さない, 1:NULL値を許す, 2:不明）
	*/
	public DbColumn(
						String columnName,
						String columnLabel,
						int columnType,
						int columnDisplaySize,
						int precision,
						int scale,
						int nullable
					)
	{
		this.szName = columnName;
		this.szLabel = columnLabel;
		this.nType = columnType;
		this.nSize = columnDisplaySize;
		this.nPrec = precision;
		this.nScale = scale;
		this.nNull = nullable;
	}

	/** コンストラクタ */
	public DbColumn( ResultSetMetaData rsmd, int nCol )
		throws SQLException
	{
		this.szName = rsmd.getColumnName( nCol );
		this.szLabel = rsmd.getColumnLabel( nCol );
		this.szClassName = rsmd.getColumnClassName( nCol );
		this.szTypeName = rsmd.getColumnTypeName( nCol );
		this.nType = rsmd.getColumnType( nCol );
		this.nSize = rsmd.getColumnDisplaySize( nCol );
		this.nPrec = rsmd.getPrecision( nCol );
		this.nScale = rsmd.getScale( nCol );
		this.nNull = rsmd.isNullable( nCol );
		//this.szTbl = rsmd.getTableName( nCol );
	}


	/**
	* Cloneable をオーバーライドします。
	* オブジェクトのクラスが Cloneable インタフェースをサポートしていない場合。
	* clone メソッドをオーバーライドしたサブクラスも、インスタンスを複製できない
	* ことを示すためにこの例外をスローすることがある
	* @return このインスタンスの複製
	* @exception CloneNotSupportedException
	* @exception OutOfMemoryError 十分なメモリがない場合
	*/
	public Object clone()
		throws CloneNotSupportedException, OutOfMemoryError
	{
		return super.clone();
	}

}


