package jp.co.sds_corp.util.file;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;

/**
* ファイル出力共通オブジェクト
* @author 小見山　聡
*/
public class CmnFileWriter
{
	PrintWriter writer;

	public CmnFileWriter() {
	}

	public void open( String absolutePath ) throws IOException {
		try {
			FileWriter w = new FileWriter( absolutePath );
			writer = new PrintWriter( new BufferedWriter( w ));
		} catch ( IOException ie ) {
			throw( ie );
		}
	}

	public Charset createCharset( String charsetName ) throws Exception {
		return Charset.forName( charsetName );
	}

	public void write( String text ) throws IOException {
		writer.write( text );
	}

	public void println( String text ) throws IOException, Exception {
		writer.write( text );
		writer.write( "\n" );
	}

	public void println( String text, String charsetName )
		throws IOException, Exception
	{
		if ( charsetName.length() <= 0 ) {
			println( text );
			return;
		}

		Charset charset = createCharset( charsetName );
		byte [] b = new byte[text.getBytes(charset.name()).length];

		b = text.getBytes( charset.name() );
		ByteBuffer bBuff = ByteBuffer.wrap( b );
		CharsetDecoder decoder = charset.newDecoder();
		CharBuffer textChars = null;
		textChars = decoder.decode( bBuff );

		Charset utfCharset = createCharset( "UTF-16LE" );
		CharsetEncoder encoder = utfCharset.newEncoder();
		ByteBuffer utfBytes = null;
		utfBytes = encoder.encode( textChars );

		byte newText[] = utfBytes.array();
		StringBuilder convertText = new StringBuilder();
		for ( int i=0,n=newText.length; i<n; i+=2 ) {
			convertText.append( getString( newText, i ));
		}

		writer.write( convertText.toString() );
		writer.write( "\n" );
	}

	public String getString( byte [] b, int offset ) throws Exception {
		if ( b[offset+1] == 0 ) {
			return new String( b, offset, 1 );
		} else {
			String b0 = Integer.toHexString(((int )b[offset+1] & 0x000000ff));
			String b1 = Integer.toHexString(((int )b[offset  ] & 0x000000ff));
			String high = "\\u" + "00".substring( 0, 2-b0.length()) + b0
							    + "00".substring( 0, 2-b1.length()) + b1;
			return high;
		}
	}

	public void close() throws IOException {
		writer.close();
	}

}
