package	jp.co.sds_corp.util.log;

import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

/**
* ログを出力します。
* @author	小見山　聡
*/
public class JdkLogger implements CmnLogger
{
	private static ConsoleLogger console = ConsoleLogger.getInstance();
	private static JdkLogger instance = new JdkLogger();
	Logger rootLogger;
	boolean needCaller;

	private JdkLogger() {
	}

	public static JdkLogger getInstance() {
		return instance;
	}

	Logger getLogger(Object obj) {
		Logger ret = null;
		String category = getCategory(obj);
		try {
			ret = Logger.getLogger(category);
		} catch (NoClassDefFoundError ex) {
		} catch (Exception ex) {
			throw(new RuntimeException("logger not found.", ex));
		}
		return ret;
	}

	public String getCategory(Object obj) {
		String ret = null;
		if (obj instanceof Class) {
			ret = ((Class<?> )obj).getName();
		} else if (obj instanceof String) {
			ret = (String )obj;
		} else {
			ret = obj.getClass().getName();
		}
		return ret;
	}

	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	level	log の出力レベル
	 */
	@Override
	public boolean isLoggable( Object obj, Level level ) {
		Logger logger = getLogger(obj);
		return logger.isLoggable(level);
	}

	/**
	 * @param	obj		log の出力を行なったClass。
	 */
	@Override
	public boolean isTraceEnable( Object obj ) {
		return isLoggable( obj, Level.FINEST );
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 */
	@Override
	public boolean isDebugEnable( Object obj ) {
		return isLoggable( obj, Level.FINER );
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 */
	@Override
	public boolean isDetailEnable( Object obj ) {
		return isLoggable( obj, Level.FINE );
	}

	/**
	* log を出力します。
	* @param	rec		出力するメッセージ
	*/
	public void log( Logger logger, LogRecord rec ) {
		if ( logger == null ) { // || isClosed() ) {
			console.log( this, rec.getLevel().toString() + ":" + rec.getMessage() );
			return;
		}
		if ( !logger.isLoggable( rec.getLevel() )) {
			return;
		}

		rec.setLoggerName( logger.getName() );
		rec.setThreadID( (int )Thread.currentThread().getId() );
/*
		rec.getSourceMethodName();
*/
		if ( needCaller ) {
			if ( rec instanceof CmnLogRecord ) {
				CmnLogRecord irec = (CmnLogRecord )rec;
				irec.setNeedCaller( true );
			}
		}

		String ebname = getEffectiveResourceBundleName( logger );
		if ( ebname != null ) {
			rec.setResourceBundleName( ebname );
			rec.setResourceBundle( findResourceBundle( ebname ));
		}
		logger.log( rec );
	}

	/**
	* log を出力します。
	* @param	level	log の出力レベル
	* @param	msg		出力するメッセージ
	*/
	public void log( Object obj, Level level, String msg, Object... arguments ) {
		Logger logger = getLogger(obj);
		CmnLogRecord rec = new CmnLogRecord( level, msg );
		rec.setLoggerName( logger.getName() );

		String ebname = getEffectiveResourceBundleName( logger );
		if ( ebname != null ) {
			rec.setResourceBundleName( ebname );
			rec.setResourceBundle( findResourceBundle( ebname ));
		}
		log( logger, rec );
	}

	private String getEffectiveResourceBundleName( Logger logger ) {
		Logger target = logger;
		while (target != null) {
			String rbn = target.getResourceBundleName();
			if (rbn != null) {
				return rbn;
			}
			target = target.getParent();
		}
		return null;
	}

	// 処理内容は簡略化
	private synchronized ResourceBundle findResourceBundle( String name ) {
		ResourceBundle ret = null;
		try {
			ret = ResourceBundle.getBundle( name );
		} catch ( Exception ex ) {
		}
		return ret;
	}

	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void trace( Object obj, String logMsg ) {
		Level level = Level.FINEST;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void trace( Object obj, String logMsg, Throwable th ) {
		Level level = Level.FINEST;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void debug( Object obj, String logMsg ) {
		Level level = Level.FINER;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void debug( Object obj, String logMsg, Throwable th ) {
		Level level = Level.FINER;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void detail( Object obj, String logMsg ) {
		Level level = Level.FINE;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void detail( Object obj, String logMsg, Throwable th ) {
		Level level = Level.FINE;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void info( Object obj, String logMsg ) {
		Level level = Level.INFO;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void info( Object obj, String logMsg, Throwable th ) {
		Level level = Level.INFO;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void warn( Object obj, String logMsg ) {
		Level level = Level.WARNING;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void warn( Object obj, String logMsg, Throwable th ) {
		Level level = Level.WARNING;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void error( Object obj, String logMsg ) {
		Level level = Level.SEVERE;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void error( Object obj, String logMsg, Throwable th ) {
		Level level = Level.SEVERE;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 */
	@Override
	public void fatal( Object obj, String logMsg ) {
		Level level = Level.SEVERE;
		log(obj, level, logMsg);
	}
	/**
	 * @param	obj		log の出力を行なったClass。
	 * @param	logMsg	出力するメッセージ
	 * @param	t		出力する例外
	 */
	@Override
	public void fatal( Object obj, String logMsg, Throwable th ) {
		Level level = Level.SEVERE;
		String error = "null";
		if (th != null) {
			error = th.getMessage();
		}
		log(obj, level, logMsg + ":" + error);
		exception(obj, level, th);
	}

	/**
	* exception log を出力します。
	* @param	ex		出力する例外
	*/
	@Override
	public void exception( Object obj, Throwable ex ) {
		Level level = Level.SEVERE;
		exception( obj, level, ex );
	}

	/**
	* exception log を出力します。
	* @param	level	log の出力レベル
	* @param	ex		出力する例外
	*/
	@Override
	public void exception( Object obj, Level level, Throwable ex ) {
		String category = "jp.co.sds_corp.EXCEPTION" + "." + getCategory(obj);
		if (ex == null ) {
			// ex.printStackTrace();
			return;
		}
		for ( ;; ) {
			StackTraceElement[] stacks = ex.getStackTrace();
			for ( StackTraceElement stack : stacks ) {
				log( category, level, stack.toString() );
			}
			ex = ex.getCause();
			if ( ex == null ) {
				break;
			}
			log( category, level, "Caused by: " + ex.toString() );
		}
	}

	/**
	 * @param	obj		log の出力を行なったClass。
	 */
	@Override
	public void sql( Object obj, String sql ) {
		Level level = Level.FINE;
		String category = "jp.co.sds_corp.SQL" + "." + getCategory(obj);
		log(category, level, sql);
	}

	private CallerInfo getCallerInfo() {
		StackTraceElement stack[] = (new Throwable()).getStackTrace();
		int ix = 0;
		while ( ix < stack.length ) {
			StackTraceElement frame = stack[ix];
			String cname = frame.getClassName();
			if ( cname.equals( "jp.co.sds_corp.util.common.CmnLogger" )) {
				break;
			}
			ix++;
		}
		while ( ix < stack.length ) {
			StackTraceElement frame = stack[ix];
			String cname = frame.getClassName();
			if ( !cname.equals( "jp.co.sds_corp.util.common.CmnLogger" )) {
				CallerInfo ret = new CallerInfo( stack[ix] );
				return ret;
			}
			ix++;
		}
		return null;
	}

	static class CallerInfo {
		String	classname;
		String	methodname;

		CallerInfo( StackTraceElement stack ) {
			classname = stack.getClassName();
			methodname = stack.getMethodName()
					+ "(" + stack.getLineNumber() + ")";
		}

		String	getClassName() {
			return classname;
		}
		String	getMethodName() {
			return methodname;
		}
	}

}
