package	jp.co.sds_corp.util.math;

/**
* 不完全ガンマ関数
*/
public class Igamma
{
	private static double LOG_PI = Math.log(Math.PI);

	public static double p_gamma(double a, double x, double loggamma_a)
		throws UtilMathException
	{
		if (x >= 1 + a) {
			return 1 - q_gamma(a, x, loggamma_a);
		}
		if (x == 0) {
			return 0;
		}
		double result = Math.exp(a * Math.log(x) - x - loggamma_a) / a;
		double term   = result;	
		for (int k = 1; k<1000; k++) {
			term *= x / (a + k);
			double previous = result;
			result += term;
			if (result == previous) {
				return result;
			}
		}
		throw new UtilMathException(UtilMathException.IGAMMA_EXCEPTION, 
					"収束しません.", new Exception("収束しません"));
	}

	public static double q_gamma(double a, double x, double loggamma_a)
		throws UtilMathException
	{
		if (x < 1 + a) {
			return 1 - p_gamma(a, x, loggamma_a);
		}
		double w = Math.exp(a * Math.log(x) - x - loggamma_a);
		double la = 1, lb = 1 + x - a;
		double result = w / lb;
		for (int k=2; k<1000; k++) {
			double temp = ((k - 1 - a) * (lb - la) + (k + x) * lb) / k;
			la = lb;
			lb = temp;
			w *= (k - 1 - a) / k;
			temp = w / (la * lb);
			double previous = result;
			result += temp;
			if (result == previous) {
				return result;
			}
		}
		throw new UtilMathException(UtilMathException.IGAMMA_EXCEPTION, 
				"収束しません.", new Exception("収束しません"));
	}

	/**
	* 標準正規分布の下側確立
	*/
	public static double p_normal(double x) throws UtilMathException {
		if (x >= 0) {
			return 0.5 * (1 + p_gamma(0.5, 0.5 * x * x, LOG_PI / 2));
		} else {
			return 0.5 * q_gamma(0.5, 0.5 * x * x, LOG_PI / 2);
		}
	}

	/**
	* 標準正規分布の上側確立
	*/
	public static double q_normal(double x) throws UtilMathException {
		if (x >= 0) {
			return 0.5 * q_gamma(0.5, 0.5 * x * x, LOG_PI / 2);
		} else {
			return 0.5 * (1 + p_gamma(0.5, 0.5 * x * x, LOG_PI / 2));
		}
	}

	public static double erf(double x) throws UtilMathException {
		if (x >= 0) {
			return p_gamma(0.5, x * x, LOG_PI / 2);
		} else {
			return -p_gamma(0.5, x * x, LOG_PI / 2);
		}
	}

	public static double erfc(double x) throws UtilMathException {
		if (x >= 0) {
			return q_gamma(0.5, x * x, LOG_PI / 2);
		} else {
			return 1 + p_gamma(0.5, x * x, LOG_PI / 2);
		}
	}

	public static double komi_normal(double x) {
        double pi2 = 1 / Math.sqrt( 2 * Math.PI );

        double is = -1;
        double y = Math.abs(x);
        double c = y*y;
        double p = 0.0;
        double z = Math.exp(-c*0.5)*pi2;

        if (y < 2.5) {
            for (int i = 20; i > 0; i--) {
                p = i*c/(i*2+1+is*p);
                is = -is;
            }
            p = 0.5-z*y/(1.0-p);
        } else {
            for (int i = 20; i > 0; i--) {
                p = i/(y+p);
            }
            p = z/(y+p);
        }
        p =  (x < 0.0) ? p : 1.0 - p;
        return p;
    }

    public static double komi_normsinv(double p) {
        double a1 =  0.161614603871299e-15;
        double b1 = -0.178837229464158e-13;
        double c1 =  0.888943322341436e-12;
        double d1 = -0.261195052190738e-10;
        double e1 =  0.500533655324426e-09;
        double f1 = -0.646627744010232e-08;
        double g1 =  0.553751002988654e-07;
        double h1 = -0.283629174340630e-06;
        double i1 =  0.539877747512783e-06;
        double j1 =  0.122127251950965e-05;
        double k1 =  0.154574365345128e-04;
        double l1 = -0.234768842092152e-03;
        double m1 = -0.830686207721635e-03;
        double n1 =  0.370684581838722e-01;
        double o1 =  1.57079636931376;
        double a2 =  0.475993226875073e-20;
        double b2 = -0.160810158951050e-17;
        double c2 =  0.247926693002221e-15;
        double d2 = -0.231079228290330e-13;
        double e2 =  0.145374571956651e-11;
        double f2 = -0.652519616362467e-10;
        double g2 =  0.215245004110748e-08;
        double h2 = -0.528966690546445e-07;
        double i2 =  0.968810841143746e-06;
        double j2 = -0.130151504271833e-04;
        double k2 =  0.122266235739026e-03;
        double l2 = -0.686950369052696e-03;
        double m2 =  0.323268032864241e-03;
        double n2 =  0.354191456970913e-01;
        double o2 =  1.57180584930853;
        double y = -Math.log(4.0*p*(1.0-p));

        double ans = (p >= 3e-7)
                ? Math.sqrt(((((((((((((((y*a1+b1)*y+c1)*y+d1)*y+e1)*y+f1)*y+g1)*y+h1)*y+i1)*y+j1)*y
+k1)*y+l1)*y+m1)*y+n1)*y+o1)*y)
                : Math.sqrt(((((((((((((((y*a2+b2)*y+c2)*y+d2)*y+e2)*y+f2)*y+g2)*y+h2)*y+i2)*y+j2)*y
+k2)*y+l2)*y+m2)*y+n2)*y+o2)*y);

        ans = (p >= 0.5) ? ans : -ans;
        return ans;
    }

    public static double bi_normal(double a, double b, double c)
		throws UtilMathException
	{
        double ans = 0.0D;
        if(c == 1.0D)
            ans = p_normal(Math.min(a, b));
        else
        if(c == -1D) {
            if(a <= -b)
                ans = 0.0D;
            else
                ans = p_normal(a) - p_normal(-b);
        } else
        if(c == 0.0D)
            ans = p_normal(a) * p_normal(b);
        else
        if(a <= 0.0D && b <= 0.0D && c <= 0.0D)
            ans = bi_normalsub(a, b, c);
        else
        if(a >= 0.0D && b >= 0.0D && c <= 0.0D)
            ans = ((p_normal(a) + p_normal(b)) - 1.0D) + bi_normalsub(-a, -b, c);
        else
        if(a <= 0.0D && b >= 0.0D && c >= 0.0D)
            ans = p_normal(a) - bi_normalsub(a, -b, -c);
        else
        if(a >= 0.0D && b <= 0.0D && c >= 0.0D)
            ans = p_normal(b) - bi_normalsub(-a, b, -c);
        else
        if(a * b * c >= 0.0D) {
            double c1 = ((c * a - b) * sign(a)) / Math.sqrt((a * a - 2D * c * a * b) + b * b);
            double c2 = ((c * b - a) * sign(b)) / Math.sqrt((a * a - 2D * c * a * b) + b * b);
            ans = (bi_normal(a, 0.0D, c1) + bi_normal(0.0D, b, c2)) - (1.0D - sign(a) * sign(b)) / 4D;
        }
        return ans;
    }
    public static double bi_normalsub(double s_a, double s_b, double s_c) {
        double a[] = {
            0.32530300000000001D,
			0.42110710000000001D,
			0.13344249999999999D,
			0.0063743230000000003D
        };
        double b[] = {
            0.13377639999999999D,
			0.62432469999999995D,
			1.3425377999999999D,
			2.2626645000000001D
        };
        double mm = 0.0D;
        double aa = s_a / Math.sqrt(2D * (1.0D - s_c * s_c));
        double bb = s_b / Math.sqrt(2D * (1.0D - s_c * s_c));
        for(int i = 0; i < a.length; i++) {
            for(int j = 0; j < a.length; j++) {
                double x = Math.exp(aa * (2D * b[i] - aa) + bb * (2D * b[j] - bb) + 2D * s_c * (b[i]
 - aa) * (b[j] - bb));
                double m = a[i] * a[j] * x;
                mm += m;
            }

        }
        return (Math.sqrt(1.0D - s_c * s_c) / 3.1415926535897931D) * mm;
    }

    private static double sign(double a) {
        if(a > 0.0D) return -1D;
        return a != 0.0D ? 1.0D : 0.0D;
    }
}
