package	jp.co.sds_corp.util.net;

import java.io.FileDescriptor;
import java.io.IOException;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketOptions;

/**
* 全てのソケット実装の基底クラスです。
*/
public abstract class RawSocketImpl implements SocketOptions
{
	protected FileDescriptor fd;
	protected int protocol;

	/**
	* 指定されたプロトコルでソケットを初期化します。
	* @param	protocol	プロトコル
	*/
	protected abstract void create( int protocol ) throws SocketException;

	/**
	* ソケットをローカルアドレスにバインドします。
	* @param	laddr	バインドするローカルアドレス
	*/
	protected abstract void bind( InetAddress laddr ) throws SocketException;

	/**
	* パケットを送信します。
	* @param	packet	送信するパケット
	*/
	protected abstract void send( RawPacket packet ) throws IOException;

	/**
	* パケットを受信します。
	* @param	packet	受信データの格納先
	*/
	protected abstract void receive( RawPacket packet ) throws IOException;

	/**
	* ソケットを閉じます。
	*/
	protected abstract void close();

	/**
	* ソケットのファイルデスクリプタを返します。
	*/
	protected FileDescriptor getFileDescriptor() {
		return fd;
	}

	/**
	* ソケットのプロトコルを返します。
	*/
	protected int getProtocol() {
		return protocol;
	}

}
