package	jp.co.sds_corp.util.script;

import java.io.File;

import jp.co.sds_corp.util.common.CmnProperty;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* ScriptParserの基底クラスです。
*/
public abstract class AbstractScriptParser implements IScriptParser
{
	IConfiguration config;
	String	language;	// language
	CmnProperty	property;	// 設定
	String	charset;	// charset
	String	script;	// スクリプト
	File	file;	// スクリプトファイル
	boolean	debug;	// デバッグフラグ
	boolean	loadOnInit = true;	// 初期化時にコンパイルフラグ
	boolean	compiled = false;	// コンパイルフラグ

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		this.config = config;
		this.language = config.getAttributes().get( "language" );
		setCharset( config.getAttributes().get( "charset", "UTF-8" ));
		String file = config.getAttributes().get( "file", null );
		if ( file != null ) {
			if ( getProperty() != null ) {
				file = getProperty().replaceValue( file );
			}
			setFile( new File( file ));
		}
		try {
			setScript( config.get());
		} catch ( Exception ex ) {
		}
		String flg;
		flg = config.getAttributes().get( "loadOnInit", "" );
		if ( flg.equals( "false" )) {
			loadOnInit = false;
		}
		flg = config.getAttributes().get( "debug", "" );
		if ( flg.equals( "true" )) {
			debug = true;
		}
	}

	/**
	* languageを設定します。
	* @param	var		設定するデータ
	*/
	public void setLanguage( String var ) {
		this.language = var;
	}
	/**
	* languageを取得します。
	* @return	language
	*/
	public String getLanguage() {
		return language;
	}

	/**
	* 設定を設定します。
	* @param	var		設定するデータ
	*/
	public void setProperty( CmnProperty var ) {
		this.property = var;
	}
	/**
	* 設定を取得します。
	* @return	設定
	*/
	public CmnProperty getProperty() {
		return property;
	}

	/**
	* charsetを設定します。
	* @param	var		設定するデータ
	*/
	public void setCharset( String var ) {
		this.charset = var;
	}
	/**
	* charsetを取得します。
	* @return	charset
	*/
	public String getCharset() {
		return charset;
	}

	/**
	* スクリプトを設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setScript( String var ) {
		this.script = var;
		compiled = false;
	}
	/**
	* スクリプトを取得します。
	* @return	スクリプト
	*/
	public synchronized String getScript() {
		return script;
	}

	/**
	* スクリプトファイルを設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setFile( File var ) {
		this.file = var;
		compiled = false;
	}
	/**
	* スクリプトファイルを取得します。
	* @return	スクリプトファイル
	*/
	public synchronized File getFile() {
		return file;
	}

	/**
	* デバッグフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var;
	}
	/**
	* デバッグフラグを取得します。
	* @return	デバッグフラグ
	*/
	public boolean isDebug() {
		return debug;
	}

	/**
	* 初期化時にコンパイルフラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setLoadOnInit( boolean var ) {
		this.loadOnInit = var;
	}
	/**
	* 初期化時にコンパイルフラグを取得します。
	* @return	初期化時にコンパイルする
	*/
	public boolean isLoadOnInit() {
		return loadOnInit;
	}

	/**
	* コンパイルフラグを設定します。
	* @param	var		設定するデータ
	*/
	public synchronized void setCompiled( boolean var ) {
		this.compiled = var;
	}
	/**
	* コンパイルフラグを取得します。
	* @return	コンパイルフラグ
	*/
	public synchronized boolean isCompiled() {
		return compiled;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config  設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		config.setAttribute( "language", getLanguage() );
		config.setAttribute( "charset", getCharset() );
		if ( getFile() != null ) {
			config.setAttribute( "file", getFile().getPath() );
		}
		if ( getScript() != null ) {
			config.set( getScript() );
		}
		if ( !isLoadOnInit()) {
			config.setAttribute( "loadOnInit", "false" );
		}
		if ( isDebug()) {
			config.setAttribute( "debug", "true" );
		}
	}

}
